
#ifdef __cplusplus
extern "C"
{
 #include "wdm.h"
}
#else
 #include "wdm.h"
#endif

typedef struct 
{
	PDEVICE_OBJECT	NextStackDevice;
}
DevicePCI_DEVICE_EXTENSION, *PDevicePCI_DEVICE_EXTENSION;

/////////////////////////////////////////////////////////////////////////////
	
NTSTATUS DevicePCI_PnP(IN PDEVICE_OBJECT fdo, IN PIRP IRP)
{
	PDevicePCI_DEVICE_EXTENSION dx = (PDevicePCI_DEVICE_EXTENSION)fdo->DeviceExtension;
	PIO_STACK_LOCATION IrpStack = IoGetCurrentIrpStackLocation(IRP);
	ULONG MinorFunction = IrpStack->MinorFunction;

	switch(MinorFunction)
	{
	case IRP_MN_START_DEVICE:
		// we should check the allocated resource...
		break;
	case IRP_MN_REMOVE_DEVICE:
		if(dx->NextStackDevice) IoDetachDevice(dx->NextStackDevice);
		IoDeleteDevice(fdo);
		break;
	}

	// call the device below us
	IoSkipCurrentIrpStackLocation(IRP);
	return IoCallDriver(dx->NextStackDevice, IRP);
}

NTSTATUS DevicePCI_AddDevice(IN PDRIVER_OBJECT DriverObject, IN PDEVICE_OBJECT pdo)
{
	// Create the device and allocate the "Device Extension"
	PDEVICE_OBJECT fdo;
	NTSTATUS status = IoCreateDevice(DriverObject, sizeof(DevicePCI_DEVICE_EXTENSION), NULL, FILE_DEVICE_UNKNOWN, 0, FALSE, &fdo);
	if(!NT_SUCCESS(status)) return status;

	// Attach to the driver below us
	PDevicePCI_DEVICE_EXTENSION dx = (PDevicePCI_DEVICE_EXTENSION)fdo->DeviceExtension;
	dx->NextStackDevice = IoAttachDeviceToDeviceStack(fdo, pdo);

	fdo->Flags &= ~DO_DEVICE_INITIALIZING;
	return STATUS_SUCCESS;
}

extern "C"
NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath)
{
 	DriverObject->DriverExtension->AddDevice = DevicePCI_AddDevice;
	DriverObject->MajorFunction[IRP_MJ_PNP] = DevicePCI_PnP;

	return STATUS_SUCCESS;
}
